@@ -9,6 +9,7 @@ use App::Bondage::Away;
 use App::Bondage::Client;
 use App::Bondage::Recall;
 use Digest::MD5 qw(md5_hex);
+use File::Spec;
 use POE qw(Filter::Line Filter::Stackable Wheel::ReadWrite Wheel::SocketFactory);
 use POE::Filter::IRCD;
 use POE::Component::Client::DNS;
@@ -22,14 +23,15 @@ use POE::Component::IRC::Plugin::NickReclaim;
 use POE::Component::IRC::Plugin::NickServID;
 use Socket qw(inet_ntoa);
 
-our $VERSION    = '0.2.5';
-our $HOMEPAGE   = 'http://search.cpan.org/dist/App-Bondage';
+our $VERSION    = '0.3.0';
+our $HOMEPAGE   = 'http://search.cpan.org/perldoc?App::Bondage';
 our $CRYPT_SALT = 'erxpnUyerCerugbaNgfhW';
 
 sub new {
     my ($package, %params) = @_;
     my $self = bless \%params, $package;
     $self->_load_config();
+    
     POE::Session->create(
         object_states => [
             $self => [ qw(_start _client_error _client_input _listener_accept _listener_failed _exit) ],
@@ -40,8 +42,14 @@ sub new {
 
 sub _start {
     my $self = $_[OBJECT];
-    $self->{filter} = POE::Filter::Stackable->new( Filters => [ POE::Filter::Line->new(), POE::Filter::IRCD->new() ] );
+    
     $self->{resolver} = POE::Component::Client::DNS->spawn();
+    $self->{filter} = POE::Filter::Stackable->new(
+        Filters => [
+            POE::Filter::Line->new(),
+            POE::Filter::IRCD->new()
+        ]
+    );
     
     while (my ($network_name, $network) = each %{ $self->{config}->{networks} }) {
         my $irc = $self->{ircs}->{$network_name} = POE::Component::IRC::State->spawn(
@@ -60,24 +68,29 @@ sub _start {
             Raw          => 1,
         );
         
-        $irc->plugin_add('CTCP',        POE::Component::IRC::Plugin::CTCP->new( Version => "Bondage $VERSION running on $Config{osname} $Config{osvers} -- $HOMEPAGE" ));
+        $irc->plugin_add('CTCP',        POE::Component::IRC::Plugin::CTCP->new(
+                Version => "Bondage $VERSION running on $Config{osname} $Config{osvers} -- $HOMEPAGE"
+        ));
         $irc->plugin_add('Cycle',       POE::Component::IRC::Plugin::CycleEmpty->new()) if $network->{cycle_empty};
         $irc->plugin_add('NickReclaim', POE::Component::IRC::Plugin::NickReclaim->new());
         $irc->plugin_add('Connector',   POE::Component::IRC::Plugin::Connector->new( Delay => 120 ));
         $irc->plugin_add('AutoJoin',    POE::Component::IRC::Plugin::AutoJoin->new(
                                             Channels => $network->{channels},
                                             RejoinOnKick => $network->{kick_rejoin} ));
+        
         if (defined $network->{nickserv_pass}) {
             $irc->plugin_add('NickServID', POE::Component::IRC::Plugin::NickServID->new(
                                             Password => $network->{nickserv_pass}, ));
         }
+        
         if ($network->{log_public} || $network->{log_private}) {
-            my $log_dir = $self->{Work_dir} . '/logs';
+            my $log_dir = File::Spec->catdir($self->{Work_dir}, 'logs');
             if (! -d $log_dir) {
-                mkdir $log_dir, oct 700 or croak "Cannot create directory $log_dir $!; aborted";
+                mkdir $log_dir or die "Cannot create directory $log_dir; $!; aborted";
             }
+
             $irc->plugin_add('Logger', POE::Component::IRC::Plugin::Logger->new(
-                                           Path         => "$log_dir/$network_name",
+                                           Path         => File::Spec->catdir($log_dir, $network_name),
                                            Private      => $network->{log_private},
                                            Public       => $network->{log_public},
                                            Sort_by_date => $network->{log_sortbydate},
@@ -88,15 +101,12 @@ sub _start {
         $irc->plugin_add('Away',   App::Bondage::Away->new( Message => $network->{away_msg}));
         $irc->plugin_add('Recall', App::Bondage::Recall->new( Mode => $network->{recall_mode} ));
 
-        $irc->yield(register => 'all');
         $irc->yield(connect => { });
     }
     
     $self->_spawn_listener();
-#    $poe_kernel->sig(HUP  => '_reload');
-#    $poe_kernel->sig(INT  => '_exit');
-#    $poe_kernel->sig(TERM => '_exit');
-#    $poe_kernel->sig(DIE  => '_exit');
+    $poe_kernel->sig(INT  => '_exit');
+    #$poe_kernel->sig(HUP  => '_reload');
 
     return;
 }
@@ -125,9 +135,9 @@ sub _client_input {
             $info->{pass} = md5_hex($info->{pass}, $CRYPT_SALT) if length $self->{config}->{password} == 32;
             last AUTH unless $info->{pass} eq $self->{config}->{password};
             last AUTH unless my $irc = $self->{ircs}->{ $info->{nick} };
-            $info->{wheel}->put($info->{nick} . ' NICK :' . $irc->nick_name());
+            $info->{wheel}->put("$info->{nick} NICK :$irc->nick_name");
             $irc->plugin_add("Client_$id" => App::Bondage::Client->new( Socket => $info->{socket} ));
-            $irc->_send_event('irc_proxy_authed' => $id);
+            $irc->_send_event(irc_proxy_authed => $id);
             delete $self->{wheels}->{$id};
             return;
         }
@@ -160,12 +170,12 @@ sub _listener_accept {
 }
 
 sub _listener_failed {
-    my ($self, $error, $wheel) = @_[OBJECT, ARG2, ARG3];
-    croak "Failed to spawn listener: $error; aborted";
+    my ($self, $error) = @_[OBJECT, ARG2];
+    die "Failed to spawn listener: $error; aborted\n";
 }
 
 sub _spawn_listener {
-    my $self = shift;
+    my ($self) = @_;
     $self->{listener} = POE::Wheel::SocketFactory->new(
         BindAddress  => $self->{config}->{listen_host},
         BindPort     => $self->{config}->{listen_port},
@@ -177,10 +187,10 @@ sub _spawn_listener {
     if ($self->{config}->{listen_ssl}) {
         require POE::Component::SSLify;
         POE::Component::SSLify->import(qw(Server_SSLify SSLify_Options));
-        eval { SSLify_Options("bondage.key", "bondage.crt") };
-        croak 'Unable to load SSL key (' . $self->{Work_dir} . '/bondage.key) or certificate (' . $self->{Work_dir} . "/bondage.crt): $!; aborted" if $!;
+        eval { SSLify_Options('bondage.key', 'bondage.crt') };
+        croak "Unable to load SSL key ($self->{Work_dir}/bondage.key) or certificate ($self->{Work_dir}/bondage.crt): $!; aborted" if $@;
         eval { $self->{listener} = Server_SSLify($self->{listener}) };
-        croak "Unable to SSLify the listener: $!; aborted" if $!;
+        croak "Unable to SSLify the listener: $@; aborted" if $@;
     }
     return;
 }
@@ -190,13 +200,13 @@ sub _load_config {
 
     my $cfg = Config::Any->load_files( {
         use_ext => 1,
-        files   => [ glob($self->{Work_dir} . '/config.*') ],
+        files   => [ glob("$self->{Work_dir}/config.*") ],
     } );
     $self->{config} = ((values %{$cfg->[0]})[0]);
 
     for my $opt (qw(listen_port password)) {
         if (!defined $self->{config}->{$opt}) {
-            croak "Config option '$opt' must be defined; aborted";
+            die "Config option '$opt' must be defined; aborted\n";
         }
     }
     
@@ -216,19 +226,15 @@ sub _load_config {
 
 # die gracefully
 sub _exit {
-    my $self = shift;
-    if (defined $self->{resolver}) {
+    my ($kernel, $self) = @_[KERNEL, OBJECT];
+    if (defined $self->{listener}) {
+        delete $self->{wheels};
         delete $self->{listener};
         $self->{resolver}->shutdown();
-        delete $self->{resolver};
-        while (my ($network, $irc) = each %{ $self->{ircs} }) {
-            $irc->yield(shutdown => 'Killed by user');
-            $irc->yield(unregister => 'all');
-            delete $self->{ircs}->{network};
-        }
+        $kernel->signal($kernel, 'POCOIRC_SHUTDOWN', 'Caught interrupt');
     }
 
-    $poe_kernel->sig_handled();
+    $kernel->sig_handled();
     return;
 }
 
@@ -256,7 +262,7 @@ it will be made modular and reusable, probably as a
 L<POE::Component::IRC|POE::Component::IRC> plugin. This keeps
 the code short and (hopefully) well tested by others.
 
-=head2 RATIONALE
+=head2 Rationale
 
 I wrote Bondage because no other IRC bouncer out there fit my needs.
 Either they were missing essential features, or they were implemented
@@ -264,11 +270,9 @@ in an undesirable (if not buggy) way. I've tried to make Bondage
 stay out of your way and be as transparent as possible.
 It's supposed to be a proxy, after all.
 
-=head2 FEATURES
+=head1 FEATURES
 
-=over
-
-=item Easy setup
+=head2 Easy setup
 
 Bondage is easy to get up and running. In the configuration file,
 you just have to specify the port it will listen on, the password,
@@ -276,116 +280,108 @@ and some IRC server(s) you want Bondage to connect to. Everything
 else has sensible defaults, though you might want to use a custom
 nickname and pick some channels to join on connect.
 
-=item Logging
+=head2 Logging
 
 Bondage can log both public and private messages for you.
-All log files are in UTF-8.
+All log files are saved as UTF-8.
 
-=item Stays connected
+=head2 Stays connected
 
 Bondage will reconnect to IRC when it gets disconnected or
 the IRC server stops responding.
 
-=item Recall messages
+=head2 Recall messages
 
 Bondage can send you all the messages you missed since you detached,
 or it can send you all messages received since it connected to
 the IRC server, or neither. This feature is based on similar features
 found in miau, dircproxy, and ctrlproxy.
 
-=item Auto-away
+=head2 Auto-away
 
 Bondage will set your status to away when no clients are attached.
 
-=item Reclaim nickname
+=head2 Reclaim nickname
 
 Bondage will periodically try to change to your preferred nickname
 if it is taken.
 
-=item Flood protection
+=head2 Flood protection
 
 Bondage utilizes POE::Component::IRC's flood protection to ensure
 that you never flood yourself off the IRC server.
 
-=item NickServ support
+=head2 NickServ support
 
 Bondage can identify with NickServ for you when needed.
 
-=item Rejoin channel if kicked
+=head2 Rejoins channels if kicked
 
 Bondage can try to rejoin a channel if you get kicked from it.
 
-=item Encrypted passwords
+=head2 Encrypted passwords
 
 Bondage supports encrypted passwords in its configuration file
 for added security.
 
-=item SSL support
+=head2 SSL support
 
-You can connect to SSL-enabled IRC servers, and make B<bondage> require
+You can connect to SSL-enabled IRC servers, and make Bondage require
 SSL for client connections.
 
-=item IPv6 support
+=head2 IPv6 support
 
 Bondage can connect to IPv6 IRC servers, and also listen for client
 connections via IPv6.
 
-=item Cycle empty channels
+=head2 Cycles empty channels
 
 Bondage can cycle (part and rejoin) channels for you when they
 become empty in order to gain ops.
 
-=item CTCP replies
+=head2 CTCP replies
 
 Bondage will reply to CTCP VERSION requests when you are offline.
 
-=back
-
 =head1 CONFIGURATION
 
 The following options are recognized in the configuration file which
 can be called F<~/.bondage/config.EXT> where EXT is an extension
 recognized by L<Config::Any|Config::Any>.
 
-=over
-
-=item listen_host
+=head2 C<listen_host>
 
 (optional, default: "0.0.0.0")
 
-The host that Bondage> listens on and accepts connections from.
-This is the host you use to connect to Bondage.
+The host that Bondage accepts connections from. This is the host you
+use to connect to Bondage.
 
-=item listen_port
+=head2 C<listen_port>
 
 (required, no default)
 
 The port Bondage binds to.
 
-=item listen_ssl
+=head2 C<listen_ssl>
 
 (optional, default: false)
 
-Set this to true if you want bondage to require the use of SSL
+Set this to true if you want Bondage to require the use of SSL
 for client connections.
 More information:
 http://www.akadia.com/services/ssh_test_certificate.html
 
-=item password
+=head2 C<password>
 
 (required, no default)
 
-The password you use to connect to Bondage. If it is 32 letters,
-it is assumed to be encrypted (see C<bondage -c>);
-
-=back
+The password you use to connect to Bondage. If it is 32 characters,
+it is assumed to be encrypted (see L<C<bondage -c>|bondage/"SYNOPSIS">);
 
 B<Note:> The rest of the options are specific to the B<network>
 block they appear in.
 
-=over
-
-=item bind_host
+=head2 C<bind_host>
 
 (optional, default: "0.0.0.0")
 
@@ -393,32 +389,32 @@ The host that Bondage binds to and connects to IRC from.
 Useful if you have multiple IPs and want to choose which one
 to IRC from.
 
-=item server_host
+=head2 C<server_host>
 
 (required, no default)
 
 The IRC server you want Bondage to connect to.
 
-=item server_port
+=head2 C<server_port>
 
 (optional, default: 6667)
 
 The port on the IRC server you want to use.
 
-=item server_pass
+=head2 C<server_pass>
 
 (optional, no default)
 
 The IRC server password, if there is one.
 
-=item use_ssl
+=head2 C<use_ssl>
 
 (optional, default: false)
 
 Set this to true if you want to use SSL to communicate with
 the IRC server.
 
-=item nickserv_pass
+=head2 C<nickserv_pass>
 
 (optional, no default)
 
@@ -426,25 +422,25 @@ Your NickServ password on the IRC network, if you have one.
 Bondage will identify with NickServ with this password on connect,
 and whenever you switch to your original nickname.
 
-=item nickname
+=head2 C<nickname>
 
 (optional, default: your UNIX user name)
 
 Your IRC nick name.
 
-=item username
+=head2 C<username>
 
 (optional, default: your UNIX user name)
 
 Your IRC user name.
 
-=item realname
+=head2 C<realname>
 
 (optional, default: your UNIX real name, if any)
 
 Your IRC real name, or email, or whatever.
 
-=item channels
+=head2 C<channels>
 
 (optional, no default)
 
@@ -456,7 +452,7 @@ Here's an example in L<YAML|YAML> format:
    "chan2" : "password"
    "chan3" : ""
 
-=item recall_mode
+=head2 C<recall_mode>
 
 (optional, default: "missed")
 
@@ -468,63 +464,59 @@ the last time you detached from Bondage.
 
 "none": Bondage will not recall any channel messages.
 
-"all": Bondage will recall all channel messages, and will
-only discard them when you leave the channel (wilfully).
+"all": Bondage will recall all channel messages.
 
 B<Note>: Bondage will always recall private messages that you missed
 while you were away, regardless of this option.
 
-=item log_public
+=head2 C<log_public>
 
 (optional, default: false)
 
 Set to true if you want Bondage to log all your public messages.
-They will be saved as C<~/.bondage/logs/some_network/#some_channel.log>
+They will be saved as F<~/.bondage/logs/some_network/#some_channel.log>
 unless you set log_sortbydate to true.
 
-=item log_private
+=head2 C<log_private>
 
 (optional, default: false)
 
 Set to true if you want Bondage to log all private messages.
-They will be saved as C<~/.bondage/logs/some_network/some_nickname.log>
+They will be saved as F<~/.bondage/logs/some_network/some_nickname.log>
 unless you set log_sortbydate to true.
 
-=item log_sortbydate
+=head2 C<log_sortbydate>
 
 (optional, default: false)
 
 Set to true if you want Bondage to rotate your logs.
-E.g. a channel log file might look like C<~/.bondage/logs/some_network/#channel/2008-01-30.log>
+E.g. a channel log file might look like
+F<~/.bondage/logs/some_network/#channel/2008-01-30.log>
 
-=item log_restricted
+=head2 C<log_restricted>
 
 (optional, default: false)
 
 Set this to true if you want Bondage to restrict the read permissions
 on created log files/directories so other users won't be able to access them.
 
-=item cycle_empty
+=head2 C<cycle_empty>
 
 (optional, default: false)
 
 Set to true if you want Bondage to cycle (part and rejoin)
 opless channels if they become empty.
 
-=item kick_rejoin
+=head2 C<kick_rejoin>
 
 (optional, default: false)
 
 Set to true if you want Bondage to try to rejoin a channel (once)
 if you get kicked from it.
 
-=back
+=head1 METHODS
 
-=head1 CONSTRUCTOR
-
-=over
-
-=item C<new>
+=head2 C<new>
 
 Arguments:
 
@@ -533,20 +525,29 @@ file. This option is required.
 
 'Debug', set to 1 to enable debugging. Default is 0.
 
-=back
-
 =head1 DEPENDENCIES
 
 The following CPAN distributions are required:
 
- Config-Any
- POE
- POE-Component-Client-DNS
- POE-Component-Daemon
- POE-Component-IRC
- POE-Component-SSLify (only if you need SSL support)
- POE-Filter-IRCD
- Socket6 (only if you need ipv6 support)
+=over 
+
+=item L<Config-Any|Config::Any>
+
+=item L<POE|POE>
+
+=item L<POE-Component-Client-DNS|POE::Component::Client::DNS>
+
+=item L<POE-Component-Daemon|POE::Component::Daemon>
+
+=item L<POE-Component-IRC|POE::Component::IRC>
+
+=item L<POE-Component-SSLify|POE::Component::IRC> (only if you need SSL support)
+
+=item L<POE-Filter-IRCD|POE::Filter::IRCD>
+
+=item L<Socket6|Socket6> (only if you need ipv6 support)
+
+=back
 
 =head1 BUGS
 
@@ -555,14 +556,15 @@ http://rt.cpan.org/Public/Dist/Display.html?Name=App%3A%3ABondage
 
 =head1 TODO
 
-Exit cleanly when clients are attached.
-
-DCC send support.
+DCC send forwarding support.
 
 Answer common client requests like WHO/MODE without asking the server,
-so other clients won't be bothered with unnecessary traffic.
+so other clients won't be bothered with unnecessary traffic. Look into
+irssi's command redirection feature.
 
-Reload the configuration file upon being sent a SIGHUP and do something useful.
+Reload the configuration file upon being sent a SIGHUP.
+
+Add proper tests.
 
 =head1 AUTHOR
 
@@ -579,12 +581,22 @@ it under the same terms as Perl itself.
 
 Other useful IRC bouncers:
 
- http://miau.sourceforge.net
- http://znc.sourceforge.net
- http://dircproxy.securiweb.net
- http://ctrlproxy.vernstok.nl
- http://www.psybnc.at
- http://irssi.org/documentation/proxy
- http://bip.t1r.net
+=over
+
+=item L<http://miau.sourceforge.net>
+
+=item L<http://znc.sourceforge.net>
+
+=item L<http://dircproxy.securiweb.net>
+
+=item L<http://ctrlproxy.vernstok.nl>
+
+=item L<http://www.psybnc.at>
+
+=item L<http://irssi.org/documentation/proxy>
+
+=item L<http://bip.t1r.net>
+
+=back
 
 =cut
