@@ -2,7 +2,6 @@ package App::Bondage::Recall;
 
 use strict;
 use warnings;
-use Carp;
 use File::Temp qw(tempfile);
 use POE::Component::IRC::Common qw( parse_user );
 use POE::Component::IRC::Plugin qw( :ALL );
@@ -10,11 +9,11 @@ use POE::Component::IRC::Plugin::BotTraffic;
 use POE::Filter::IRCD;
 use Tie::File;
 
-our $VERSION = '1.2';
+our $VERSION = '1.3';
 
 sub new {
     my ($package, %self) = @_;
-    if (!$self{Mode} || $self{Mode} !~ /(missed|all|none)/) {
+    if (!$self{Mode} || $self{Mode} !~ /missed|all|none/) {
         $self{Mode} = 'missed';
     }
     return bless \%self, $package;
@@ -24,21 +23,22 @@ sub PCI_register {
     my ($self, $irc) = @_;
     
     if (!$irc->isa('POE::Component::IRC::State')) {
-        croak __PACKAGE__ . ' requires PoCo::IRC::State or a subclass thereof';
+        die __PACKAGE__ . " requires PoCo::IRC::State or a subclass thereof\n";
     }
     
     if (!grep { $_->isa('POE::Component::IRC::Plugin::BotTraffic') } @{ $irc->pipeline->{PIPELINE} }) {
         $irc->plugin_add('BotTraffic', POE::Component::IRC::Plugin::BotTraffic->new());
     }
     
-    $self->{irc} = $irc;
-    $self->{filter} = POE::Filter::IRCD->new();
+    $self->{irc}     = $irc;
+    $self->{filter}  = POE::Filter::IRCD->new();
     $self->{clients} = 0;
-    $self->{recall} = [ ];
-    tie @{ $self->{recall} }, 'Tie::File', scalar tempfile() if $self->{Mode} =~ /(all|missed)/;
+    $self->{recall}  = [ ];
+    
+    tie @{ $self->{recall} }, 'Tie::File', scalar tempfile() if $self->{Mode} =~ /all|missed/;
+    
     $irc->raw_events(1);
-  
-    $irc->plugin_register($self, 'SERVER', qw(bot_ctcp_action bot_public connected ctcp_action msg part proxy_authed proxy_close raw));
+    $irc->plugin_register($self, 'SERVER', qw(bot_ctcp_action bot_public connected ctcp_action msg public part proxy_authed proxy_close raw));
     return 1;
 }
 
@@ -50,10 +50,10 @@ sub PCI_unregister {
 
 sub S_bot_ctcp_action {
     my ($self, $irc) = splice @_, 0, 2;
-    my $recipients = join (',', @{ ${ $_[0] } });
-    my $msg = ${ $_[1] };
+    my $recipients   = join (',', @{ ${ $_[0] } });
+    my $msg          = ${ $_[1] };
     
-    if ($self->{Mode} =~ /(all|missed)/) {
+    if ($self->{Mode} =~ /all|missed/) {
         my $line = ':' . $irc->nick_long_form($irc->nick_name()) . " PRIVMSG $recipients :\x01ACTION $msg\x01";
         push @{ $self->{recall} }, $line;
     }
@@ -63,10 +63,10 @@ sub S_bot_ctcp_action {
 
 sub S_bot_public {
     my ($self, $irc) = splice @_, 0, 2;
-    my $recipients = join (',', @{ ${ $_[0] } });
-    my $msg = ${ $_[1] };
+    my $recipients   = join (',', @{ ${ $_[0] } });
+    my $msg          = ${ $_[1] };
 
-    if ($self->{Mode} =~ /(all|missed)/) {
+    if ($self->{Mode} =~ /all|missed/) {
         my $line = ':' . $irc->nick_long_form($irc->nick_name()) . " PRIVMSG $recipients :$msg";
         push @{ $self->{recall} }, $line;
     }
@@ -76,44 +76,47 @@ sub S_bot_public {
 
 sub S_connected {
     my ($self, $irc) = splice @_, 0, 2;
-    $self->{stash} = [ ];
+    
+    $self->{stash}    = [ ];
     $self->{stashing} = 1;
+    $self->{idmsg}    = 0;
     return PCI_EAT_NONE;
 }
 
 sub S_ctcp_action {
     my ($self, $irc) = splice @_, 0, 2;
-    my $sender = ${ $_[0] };
-    my $recipients = ${ $_[1] };
-    my $msg = ${ $_[2] };
+    my $sender       = ${ $_[0] };
+    my $recipients   = ${ $_[1] };
+    my $msg          = ${ $_[2] };
+
     return PCI_EAT_NONE if $self->{clients};
     
     for my $recipient (@{ $recipients }) {
         if ($recipient eq $irc->nick_name()) {
-            my $line = ":$sender PRIVMSG " . $irc->nick_name() . " :\x01ACTION$msg\x01";
-            print "saving msg: $line\n";
+            my $line = ":$sender PRIVMSG $irc->nick_name :\x01ACTION$msg\x01";
             push @{ $self->{recall} }, $line;
         }
     }
+    
     return PCI_EAT_NONE;
 }
 
 sub S_msg {
     my ($self, $irc) = splice @_, 0, 2;
-    my $sender = ${ $_[0] };
-    my $msg = ${ $_[2] };
+    my $sender       = ${ $_[0] };
+    my $msg          = ${ $_[2] };
     
     if (!$self->{clients}) {
-        my $line = ":$sender PRIVMSG " . $irc->nick_name() . " :$msg";
+        my $line = ":$sender PRIVMSG $irc->nick_name :$msg";
         push @{ $self->{recall} }, $line;
     }
-    
+
     return PCI_EAT_NONE;
 }
               
 sub S_part {
     my ($self, $irc) = splice @_, 0, 2;
-    my $chan = ${ $_[1] };
+    my $chan         = ${ $_[1] };
 
     if (my $cycle = grep { $_->isa('POE::Component::IRC::Plugin::CycleEmpty') } @{ $irc->pipeline->{PIPELINE} } ) {
         return PCI_EAT_NONE if $cycle->cycling($chan);
@@ -127,7 +130,7 @@ sub S_part {
 #            if (lc $input->[$line]->{params}->[0] eq lc $chan) {
 #                delete $self->{recall}->[$line];
 #            }
-#            elsif ($input->[$line]->{command} =~ /(332|333|366)/ && lc $input->[$line]->{params}->[1] eq lc $chan) {
+#            elsif ($input->[$line]->{command} =~ /332|333|366/ && lc $input->[$line]->{params}->[1] eq lc $chan) {
 #                delete $self->{recall}->[$line];
 #            }
 #            elsif ($input->[$line]->{command} eq '353' && lc $input->[$line]->{params}->[2] eq lc $chan) {
@@ -139,6 +142,21 @@ sub S_part {
     return PCI_EAT_NONE;
 }
 
+sub S_public {
+    my ($self, $irc) = splice @_, 0, 2;
+    my $sender       = ${ $_[0] };
+    my $chan         = ${ $_[1] }->[0];
+    my $msg          = ${ $_[2] };
+
+    # do this here instead rather than in S_raw so that IDENTIFY-MSG
+    # will by handled by POE::Filter::IRC::Compat
+    if ($self->{Mode} =~ /all|missed/) {
+        push @{ $self->{recall} }, ":$sender PRIVMSG $chan :$msg";
+    }
+
+    return PCI_EAT_NONE;
+}
+
 sub S_proxy_authed {
     my ($self, $irc) = splice @_, 0, 2;
     $self->{clients}++;
@@ -164,29 +182,25 @@ sub S_raw {
     
     if ($self->{stashing}) {
         # capture all numeric commands until we've got the MOTD
-        if ($input->{command} =~ /\d\d\d/) {
+        if ($input->{command} =~ /\d{3}/) {
             push @{ $self->{stash} }, $raw_line;
         }
         # RPL_ENDOFMOTD / ERR_NOMOTD
-        if ($input->{command} =~ /(376|422)/) {
+        if ($input->{command} =~ /376|422/) {
             $self->{stashing} = 0;
         }
     }
     
-    if ($self->{Mode} =~ /(all|missed)/) {
-        if ($input->{command} eq 'PRIVMSG' && $input->{params}->[0] =~ /^[#&+!]/) {
-            # channel messages
-            push @{ $self->{recall} }, $raw_line;
-        }
-        elsif ($input->{command} eq 'MODE' && $input->{params}->[0] =~ /^[#&+!]/) {
+    if ($self->{Mode} =~ /all|missed/) {
+        if ($input->{command} eq 'MODE' && $input->{params}->[0] =~ /^[#&+!]/) {
             # channel mode changes
             push @{ $self->{recall} }, $raw_line;
         }
-        elsif ($input->{command} =~ /(JOIN|KICK|PART|QUIT|NICK|353|366)/) {
+        elsif ($input->{command} =~ /JOIN|KICK|PART|QUIT|NICK|TOPIC|353|366/) {
             # other channel-related things
             push @{ $self->{recall} }, $raw_line;
         }
-        elsif ($input->{command} =~ /(332|333)/) {
+        elsif ($input->{command} =~ /332|333/) {
             # only log topic stuff if we were just joining the channel
             push @{ $self->{recall} }, $raw_line if !$irc->channel_list($input->{params}->[0]);
         }
@@ -195,10 +209,12 @@ sub S_raw {
     return PCI_EAT_NONE;
 }
 
+# returns everything that an IRC server would send us upon joining
+# the channels we're on
 sub _get_chaninfo {
-    my $self = shift;
-    my $irc = $self->{irc};
-    my $me = $irc->nick_name();
+    my ($self) = @_;
+    my $irc    = $self->{irc};
+    my $me     = $irc->nick_name();
     my $server = $irc->server_name();
 
     my @info;
@@ -248,14 +264,21 @@ sub _get_chaninfo {
 }
 
 sub recall {
-    my $self = shift;
-    my @lines = ();
-    
-    my $me = $self->{irc}->nick_name();
+    my ($self) = @_;
+    my $irc    = $self->{irc};
+    my $me     = $irc->nick_name();
+    my @lines;
+
     for my $line (@{ $self->{stash} }) {
-        $line =~ s/(\S+\s+\S+) +\S+ +(.*)/$1 $me $2/;
+        $line =~ s/^(\S+ +\S+) +\S+ +(.*)/$1 $me $2/;
         push @lines, $line;
     }
+    
+    # any user modes in effect?
+    if ($irc->umode()) {
+        push @lines, ":$irc->server_name MODE $me :+$irc->umode";
+    }
+    
     push @lines, @{ $self->{recall} };
 
     if ($self->{Mode} eq 'all') {
@@ -266,7 +289,6 @@ sub recall {
                 delete $self->{recall}->[$line];
             }
         }
-
     }
     elsif ($self->{Mode} eq 'missed') {
         $self->{recall} = [ ];
@@ -276,6 +298,8 @@ sub recall {
         $self->{recall} = [ ];
         push @lines, $self->_get_chaninfo();
     }
+
+    push @lines, ":$irc->server_name 290 $me :IDENTIFY-MSG" if $self->{idmsg};
     
     return @lines;
 }
@@ -304,11 +328,9 @@ This plugin requires the IRC component to be L<POE::Component::IRC::State|POE::C
 or a subclass thereof. It also requires a L<POE::Component::IRC::Plugin::BotTraffic|POE::Component::IRC::Plugin::BotTraffic>
 to be in the plugin pipeline. It will be added automatically if it is not present.
 
-=head1 CONSTRUCTOR
+=head1 METHODS
 
-=over
-
-=item C<new>
+=head2 C<new>
 
 One optional argument:
 
@@ -321,8 +343,6 @@ private messages, regardless of this option.
 Returns a plugin object suitable for feeding to L<POE::Component::IRC|POE::Component::IRC>'s
 C<plugin_add()> method.
 
-=back
-
 =head1 AUTHOR
 
 Hinrik E<Ouml>rn SigurE<eth>sson, hinrik.sig@gmail.com
